@@ -102,6 +102,21 @@ context('rtCreateTrans()')
 
   })
 
+  test_that("min_period_dist argument works", {
+
+    expect_is(rt_df <- rtCreateTrans(trans_df=sales,
+                                     prop_id='pinx',
+                                     trans_id='sale_id',
+                                     price='sale_price',
+                                     date='sale_date',
+                                     periodicity='monthly',
+                                     seq_only = TRUE,
+                                     min_period_dist = 12),
+              'rtdata')
+    expect_true(nrow(rt_df) == 3795)
+
+
+  })
   test_that("Fails if sales creation fails", {
 
     # Bad Date field
@@ -122,6 +137,9 @@ context('rtCreateTrans()')
 
   })
 
+
+
+
   # Create a sales_df in the global env for future use
   sales_df <- dateToPeriod(trans_df = sales,
                            date = 'sale_date',
@@ -193,7 +211,8 @@ context('rtTimeMatrix()')
 context('hpiModel.rtdata(): Prior to rtModel() call')
 
   test_that('hpiModel.rtdata works in simplest format',{
-    expect_is(rt_model <- hpiModel(hpi_df = rt_df,
+    expect_is(rt_model <- hpiModel(model_type = 'rt',
+                                   hpi_df = rt_df,
                                    estimator = 'base',
                                    log_dep = TRUE),
               'hpimodel')
@@ -202,12 +221,14 @@ context('hpiModel.rtdata(): Prior to rtModel() call')
   test_that('"log_dep" argument works both ways',{
 
     # TRUE
-    expect_true(hpiModel(hpi_df = rt_df,
+    expect_true(hpiModel(model_type = 'rt',
+                         hpi_df = rt_df,
                          estimator = 'base',
                          log_dep = TRUE)$model_obj$fitted.values[1] < 1)
 
     # FALSE
-    expect_true(hpiModel(hpi_df = rt_df,
+    expect_true(hpiModel(model_type = 'rt',
+                         hpi_df = rt_df,
                          estimator = 'base',
                          log_dep = FALSE)$model_obj$fitted.values[1] > 10000)
 
@@ -219,29 +240,35 @@ context('hpiModel.rtdata(): Prior to rtModel() call')
 
     # Zero Price
     rt_dfx$price_1[1] <- 0
-    expect_error(rt_model <- hpiModel(hpi_df = rt_dfx,
+    expect_error(rt_model <- hpiModel(model_type = 'rt',
+                                      hpi_df = rt_dfx,
                                       estimator = 'base',
                                       log_dep = TRUE))
-    expect_is(rt_model <- hpiModel(hpi_df = rt_dfx,
+    expect_is(rt_model <- hpiModel(model_type = 'rt',
+                                   hpi_df = rt_dfx,
                                    estimator = 'base',
                                    log_dep = FALSE),
               'hpimodel')
 
     # NA
     rt_dfx$price_1[1] <- NA_integer_
-    expect_error(rt_model <- hpiModel(hpi_df = rt_dfx,
+    expect_error(rt_model <- hpiModel(model_type = 'rt',
+                                      hpi_df = rt_dfx,
                                       estimator = 'base',
                                       log_dep = TRUE))
-    expect_error(rt_model <- hpiModel(hpi_df = rt_dfx,
+    expect_error(rt_model <- hpiModel(model_type = 'rt',
+                                      hpi_df = rt_dfx,
                                       estimator = 'base',
                                       log_dep = FALSE))
 
     # Infinity
     rt_dfx$price_1[1] <- Inf
-    expect_error(rt_model <- hpiModel(hpi_df = rt_dfx,
+    expect_error(rt_model <- hpiModel(model_type = 'rt',
+                                      hpi_df = rt_dfx,
                                       estimator = 'base',
                                       log_dep = TRUE))
-    expect_error(rt_model <- hpiModel(hpi_df = rt_dfx,
+    expect_error(rt_model <- hpiModel(model_type = 'rt',
+                                      hpi_df = rt_dfx,
                                       estimator = 'base',
                                       log_dep = FALSE))
 
@@ -250,18 +277,22 @@ context('hpiModel.rtdata(): Prior to rtModel() call')
   test_that('Check for estimator type works',{
 
     # Base
-    expect_true(hpiModel(hpi_df = rt_df)$estimator == 'base')
+    expect_true(hpiModel(model_type = 'rt',
+                         hpi_df = rt_df)$estimator == 'base')
 
     # Convert to base with bad estimator
-    expect_true(hpiModel(hpi_df = rt_df,
+    expect_true(hpiModel(model_type = 'rt',
+                         hpi_df = rt_df,
                          estimator='xxxx')$estimator == 'base')
 
     # Robust
-    expect_true(hpiModel(hpi_df = rt_df,
+    expect_true(hpiModel(model_type = 'rt',
+                         hpi_df = rt_df,
                          estimator='robust')$estimator == 'robust')
 
     # Weighted
-    expect_true(hpiModel(hpi_df = rt_df,
+    expect_true(hpiModel(model_type = 'rt',
+                         hpi_df = rt_df,
                          estimator='weighted')$estimator == 'weighted')
   })
 
@@ -368,35 +399,43 @@ context('rtModel()')
 context('hpiModel.rtdata(): after rtModel()')
 
   test_that('hpiModel.rtdata works in both trim_model cases', {
-    expect_is(rt_model <- hpiModel(hpi_df = rt_df,
+    expect_is(rt_model <- hpiModel(model_type = 'rt',
+                                   hpi_df = rt_df,
                                    estimator = 'base',
                                    log_dep = TRUE,
                                    trim_model=TRUE), 'hpimodel')
-    expect_is(rt_model <- hpiModel(hpi_df = rt_df,
+    expect_is(rt_model <- hpiModel(model_type = 'rt',
+                                   hpi_df = rt_df,
                                    estimator = 'base',
                                    log_dep = TRUE,
                                    trim_model=FALSE), 'hpimodel')
-    expect_is(rt_model <- hpiModel(hpi_df = rt_df,
+    expect_is(rt_model <- hpiModel(model_type = 'rt',
+                                   hpi_df = rt_df,
                                    estimator = 'base',
                                    log_dep = FALSE,
                                    trim_model=TRUE), 'hpimodel')
-    expect_is(rt_model <- hpiModel(hpi_df = rt_df,
+    expect_is(rt_model <- hpiModel(model_type = 'rt',
+                                   hpi_df = rt_df,
                                    estimator = 'base',
                                    log_dep = FALSE,
                                    trim_model=FALSE), 'hpimodel')
-    expect_is(rt_model <- hpiModel(hpi_df = rt_df,
+    expect_is(rt_model <- hpiModel(model_type = 'rt',
+                                   hpi_df = rt_df,
                                    estimator = 'weighted',
                                    log_dep = TRUE,
                                    trim_model=FALSE), 'hpimodel')
-    expect_is(rt_model <- hpiModel(hpi_df = rt_df,
+    expect_is(rt_model <- hpiModel(model_type = 'rt',
+                                   hpi_df = rt_df,
                                    estimator = 'robust',
                                    log_dep = FALSE,
                                    trim_model=TRUE), 'hpimodel')
-    expect_true(is.null(hpiModel(hpi_df = rt_df,
+    expect_true(is.null(hpiModel(model_type = 'rt',
+                                 hpi_df = rt_df,
                                  estimator = 'weighted',
                                  log_dep = TRUE,
                                  trim_model=TRUE)$model_obj$qr))
-    expect_true(!is.null(hpiModel(hpi_df = rt_df,
+    expect_true(!is.null(hpiModel(model_type = 'rt',
+                                  hpi_df = rt_df,
                                   estimator = 'weighted',
                                   log_dep = TRUE,
                                   trim_model=FALSE)$model_obj$qr))
@@ -405,21 +444,33 @@ context('hpiModel.rtdata(): after rtModel()')
   test_that('hpiModel.rtdata outputs are correct', {
 
     # Run a model of each estimator type
-    rt_model_base <- hpiModel(hpi_df = rt_df,
+    rt_model_base <- hpiModel(model_type = 'rt',
+                              hpi_df = rt_df,
                               estimator = 'base',
                               log_dep = TRUE,
                               trim_model=TRUE)
 
-    rt_model_robust <- hpiModel(hpi_df = rt_df,
+    rt_model_robust <- hpiModel(model_type = 'rt',
+                                hpi_df = rt_df,
                                 estimator = 'robust',
                                 log_dep = TRUE,
                                 trim_model=FALSE)
 
-    rt_model_wgt <- hpiModel(hpi_df = rt_df,
+    rt_model_wgt <- hpiModel(model_type = 'rt',
+                             hpi_df = rt_df,
                              estimator = 'weighted',
                              log_dep = FALSE,
                              trim_model=TRUE)
 
+    set.seed(123)
+    rt_model_wwgt <- hpiModel(model_type = 'rt',
+                              hpi_df = rt_df,
+                              estimator = 'weighted',
+                              log_dep = FALSE,
+                              weights = runif(nrow(rt_df), 0, 1),
+                              trim_model=TRUE)
+
+
     # Estimatort
     expect_is(rt_model_base$estimator, 'base')
     expect_is(rt_model_robust$estimator, 'robust')
@@ -432,6 +483,9 @@ context('hpiModel.rtdata(): after rtModel()')
     expect_true(nrow(rt_model_base$coefficients) == 84)
     expect_true(max(rt_model_robust$coefficients$time) == 84)
     expect_true(rt_model_wgt$coefficients$coefficient[1] == 0)
+    expect_false(identical(
+      rt_model_wgt$coefficients$coefficient,
+                  rt_model_wwgt$coefficients$coefficient))
 
     # Modelobj
     expect_is(rt_model_base$model_obj, 'rtmodel')
@@ -466,7 +520,8 @@ context('hpiModel.rtdata(): after rtModel()')
 
  context('modelToIndex')
 
- rt_model <- hpiModel(hpi_df = rt_df,
+ rt_model <- hpiModel(model_type = 'rt',
+                      hpi_df = rt_df,
                       estimator = 'base',
                       log_dep = TRUE,
                       trim_model=TRUE)
@@ -480,25 +535,29 @@ context('hpiModel.rtdata(): after rtModel()')
   test_that('modelToIndex works with other estimatort and options', {
 
     # Robust, LogDep=T, TrimModel=T
-    expect_is(modelToIndex(hpiModel(hpi_df = rt_df,
+    expect_is(modelToIndex(hpiModel(model_type = 'rt',
+                                    hpi_df = rt_df,
                                     estimator = 'robust',
                                     log_dep = TRUE,
                                     trim_model=TRUE)), 'hpiindex')
 
     # Weighted, LogDep=T, TrimModel=T
-    expect_is(modelToIndex(hpiModel(hpi_df = rt_df,
+    expect_is(modelToIndex(hpiModel(model_type = 'rt',
+                                    hpi_df = rt_df,
                                     estimator = 'weighted',
                                     log_dep = TRUE,
                                     trim_model=TRUE)), 'hpiindex')
 
     # Robust, LogDep=F, TrimModel=T
-    expect_is(modelToIndex(hpiModel(hpi_df = rt_df,
+    expect_is(modelToIndex(hpiModel(model_type = 'rt',
+                                    hpi_df = rt_df,
                                     estimator = 'robust',
                                     log_dep = FALSE,
                                     trim_model=TRUE)), 'hpiindex')
 
     # Weighted, LogDep=T, TrimModel=F
-    expect_is(modelToIndex(hpiModel(hpi_df = rt_df,
+    expect_is(modelToIndex(hpiModel(model_type = 'rt',
+                                    hpi_df = rt_df,
                                     estimator = 'weighted',
                                     log_dep = TRUE,
                                     trim_model=FALSE)), 'hpiindex')
@@ -510,7 +569,8 @@ context('hpiModel.rtdata(): after rtModel()')
 
   test_that('modelToIndex imputes properly, BASE model, LogDEP',{
 
-    model_base <- hpiModel(hpi_df = rt_df,
+    model_base <- hpiModel(model_type = 'rt',
+                           hpi_df = rt_df,
                            estimator = 'base',
                            log_dep = TRUE,
                            trim_model=TRUE)
@@ -541,7 +601,8 @@ context('hpiModel.rtdata(): after rtModel()')
 
   test_that('modelToIndex imputes properly, BASE model, LogDep=FALSE',{
 
-    model_base <- hpiModel(hpi_df = rt_df,
+    model_base <- hpiModel(model_type = 'rt',
+                           hpi_df = rt_df,
                            estimator = 'base',
                            log_dep = FALSE,
                            trim_model=TRUE)
@@ -572,7 +633,8 @@ context('hpiModel.rtdata(): after rtModel()')
 
   test_that('modelToIndex imputes properly, Robust model, LogDEP = TRUE',{
 
-    model_base <- hpiModel(hpi_df = rt_df,
+    model_base <- hpiModel(model_type = 'rt',
+                           hpi_df = rt_df,
                            estimator = 'robust',
                            log_dep = TRUE,
                            trim_model=TRUE)
@@ -600,7 +662,8 @@ context('hpiModel.rtdata(): after rtModel()')
 
   test_that('modelToIndex imputes properly, Weighted model, LogDep=FALSE',{
 
-    model_base <- hpiModel(hpi_df = rt_df,
+    model_base <- hpiModel(model_type = 'rt',
+                           hpi_df = rt_df,
                            estimator = 'weighted',
                            log_dep = FALSE,
                            trim_model=TRUE)
@@ -628,7 +691,8 @@ context('hpiModel.rtdata(): after rtModel()')
 
   test_that('modelToIndex "max_period" cutoff works',{
 
-    model_base <- hpiModel(hpi_df = rt_df,
+    model_base <- hpiModel(model_type = 'rt',
+                           hpi_df = rt_df,
                            estimator = 'weighted',
                            log_dep = FALSE,
                            trim_model=TRUE)
@@ -644,7 +708,8 @@ context('hpiModel.rtdata(): after rtModel()')
 
   context('smoothIndex()')
 
-  model_base <- hpiModel(hpi_df = rt_df,
+  model_base <- hpiModel(model_type = 'rt',
+                         hpi_df = rt_df,
                          estimator = 'base',
                          log_dep = TRUE,
                          trim_model=TRUE)
@@ -807,6 +872,30 @@ context('rtindex() wrapper')
                          log_dep = TRUE,
                          periodicity = 'monthly'))
 
+    expect_error(rtIndex(trans_df = sales,
+                         date = 'sale_date',
+                         price = 'sale_price',
+                         prop_id = 'pinx',
+                         estimator = 'base',
+                         log_dep = TRUE,
+                         periodicity = 'monthly'))
+
+    expect_error(rtIndex(trans_df = sales,
+                         date = 'sale_date',
+                         price = 'sale_price',
+                         trans_id = 'sale_id',
+                         estimator = 'base',
+                         log_dep = TRUE,
+                         periodicity = 'monthly'))
+
+    expect_error(rtIndex(trans_df = sales,
+                         date = 'sale_date',
+                         trans_id = 'sale_id',
+                         prop_id = 'pinx',
+                         estimator = 'base',
+                         log_dep = TRUE,
+                         periodicity = 'monthly'))
+
     expect_error(rtIndex(trans_df = sales,
                          date = 'sale_date',
                          price = 'sale_price',
@@ -816,6 +905,16 @@ context('rtindex() wrapper')
                          log_dep = TRUE,
                          periodicity = 'xxx'))
 
+    expect_error(rtIndex(trans_df = sales[1, ],
+                         date = 'sale_date',
+                         price = 'sale_price',
+                         trans_id = 'sale_id',
+                         prop_id = 'pinx',
+                         estimator = 'base',
+                         log_dep = TRUE,
+                         periodicity = 'monthly'), 'Converting transactions')
+
+
   })
 
   test_that("Bad arguments generate errort: trans_df Case",{
@@ -832,6 +931,12 @@ context('rtindex() wrapper')
                          prop_id = 'pinx',
                          estimator = 'base',
                          log_dep = TRUE))
+    expect_error(rtIndex(trans_df = sales_df,
+                         price = 'sale_price',
+                         prop_id = 'pinx',
+                         estimator = 'base',
+                         log_dep = TRUE))
+
     expect_error(rtIndex(trans_df = sales_df,
                          price = 'sale_price',
                          trans_id = 'sale_id',
